// VC0706チップのカメラテスト(スナップ写真を撮るサンプル)
// ArduinoIDE 1.8.1/Arduino UNO での実験
#include <SoftwareSerial.h>
#include <SD.h>
#include <skVC0706.h>

#define SDCSpin     8    // SDカードのSPI-CS接続ピン番号
#define SWpin       7    // スイッチ接続ピン番号

int  FileNo ;       // ファイル番号
char JPG_Size[3][8] = {"640x480","320x240","160x120"} ;

SoftwareSerial mySerial = SoftwareSerial(2,3) ;   // ソフトシリアルのオブジェクト作成(RX,TX)
skVC070x  VC0706(&mySerial) ;                     // VC0706カメラのオブジェクト作成


// カメラからエラー情報が返された時に表示する処理
void ErrorDSP(char *c)
{
     union VC070X_ERROR_t e ;

     e.info = VC0706.GetError() ;  // 最後に受信したエラー情報を得る
     Serial.print(c) ;
     Serial.print("Error cmdNo=0x") ;
     Serial.print(e.cmd_no,HEX) ;  // 最後に発行したコマンド番号を表示
     Serial.print(" ErrNo=") ;
     Serial.println(e.err_no,DEC) ;// エラー番号を表示
}
/*******************************************************************************
*  電源起動時とリセットの時だけのみ処理される関数(初期化と設定処理)            *
*******************************************************************************/
void setup() {
     char *p ;
     int  ans ;

     pinMode(SWpin,INPUT_PULLUP) ; // スイッチ接続の端子、プルアップ入力で設定
     pinMode(10, OUTPUT) ;         // この行がないとSDライブラリが動作しないらしい
     
     Serial.begin(9600) ;
     // ＳＤカードの初期化処理(フォーマットではないよ)
     if (!SD.begin(SDCSpin)) {
         // カードの初期化に失敗したか、またはＳＤが入っていない
         Serial.println("SD card failed, or not present") ;
         return ;  // 終了
     }
     Serial.println("SD card initialized.") ;   // ＳＤカードの初期化完了
     // カメラの初期化処理(ソフトシリアル通信の初期化)
     VC0706.begin(38400) ;
     delay(3000) ;                 // 3秒後に開始

     // GetVersion処理
     p = VC0706.GetVersion() ;
     if (p == 0) ErrorDSP("GetVersion") ;
     else        Serial.println(p) ;
     // GetImageSize処理
     ans = VC0706.GetImageSize() ;
     if (ans == -1) {
          ErrorDSP("GetImageSize") ;
          return ;                 // 終了
     } else Serial.println(&JPG_Size[ans&0x0F][0]) ;

     FileNo = 0 ;                  // 写真番号、写真を撮る度にプラスされる
     Serial.println("<<< Now, please take a picture! >>>") ;
}
/*******************************************************************************
*  繰り返し実行される処理の関数(メインの処理)                                  *
*******************************************************************************/
void loop() {
     File fds  ;
     uint32_t time ;
     uint32_t jpgsz ;
     uint16_t adrs ;
     int i , jpgcnt , jpglst ;
     char *p , buf[18] ;

     if (digitalRead(SWpin) == LOW) {
          // スイッチが押されたら処理開始
          Serial.println("*** SwitchON ***") ;
          // ビデオを止めてキャプチャーを行う
          jpgsz = VC0706.TakePicture() ;
          if (jpgsz != 0) {
               Serial.print("File size = ") ;
               Serial.println(jpgsz,DEC) ;
               // ファイルを新規に作成してオープンする
               // ファイルが存在する場合は上書きされます
               time = millis() ;
               sprintf(buf,"IMG%05d.JPG\0",FileNo) ;
               fds = SD.open(buf,FILE_WRITE) ;
               if (fds) {
                    fds.seek(0) ;   // 先頭に位置付ける
                    jpgcnt = jpgsz / VC070X_READDATA_BUFFSZ ; // データの読込み回数
                    jpglst = jpgsz % VC070X_READDATA_BUFFSZ ; // 最後読込み回のデータ長さ
                    adrs = 0 ;
                    for(i=0 ; i<jpgcnt ; i++) {
                         p = VC0706.ReadPicture(adrs,VC070X_READDATA_BUFFSZ) ;
                         // ファイルへ書き込む
                         fds.write(p,VC070X_READDATA_BUFFSZ) ;
                         adrs = adrs + VC070X_READDATA_BUFFSZ ;
                    }
                    if (jpglst != 0) {
                         // 最後のデータをファイルへ書き込む
                         p = VC0706.ReadPicture(adrs,jpglst) ;
                         fds.write(p,jpglst) ;
                    }
                    // ファイルのクローズ
                    fds.close() ;
                    // 終了
                    time = millis() - time ;
                    Serial.print("done! ") ;
                    Serial.print(time) ;
                    Serial.println("ms") ;
                    Serial.println("--- Success ---") ;
                    FileNo++ ;
               } else {
                    // ファイルのオープンエラー
                    Serial.println("FailOpen") ;
               }
               // ビデオを再起動する
               VC0706.ResumeVideo() ;
          } else {
               // TakePictureエラー
               ErrorDSP("TakePicture") ;
          }
     }
}
