// VC0706チップのカメラテスト(動きを監視するサンプル)
// ArduinoIDE 1.8.1/Arduino UNO での実験
#include <SoftwareSerial.h>
#include <SD.h>
#include <skVC0706.h>

#define SDCSpin     8    // SDカードのSPI-CS接続ピン番号
#define SWpin       7    // スイッチ接続ピン番号

int  FileNo ;            // ファイル番号
char JPG_Size[3][8] = {"640x480","320x240","160x120"} ;

SoftwareSerial mySerial = SoftwareSerial(2,3) ;   // ソフトシリアルのオブジェクト作成(RX,TX)
skVC070x  VC0706(&mySerial) ;                     // VC0706カメラのオブジェクト作成


// カメラからエラー情報が返された時に表示する処理
void ErrorDSP(char *c)
{
     union VC070X_ERROR_t e ;

     e.info = VC0706.GetError() ;  // 最後に受信したエラー情報を得る
     Serial.print(c) ;
     Serial.print("Error cmdNo=0x") ;
     Serial.print(e.cmd_no,HEX) ;  // 最後に発行したコマンド番号を表示
     Serial.print(" ErrNo=") ;
     Serial.println(e.err_no,DEC) ;// エラー番号を表示
}
/*******************************************************************************
*  電源起動時とリセットの時だけのみ処理される関数(初期化と設定処理)            *
*******************************************************************************/
void setup() {
     char *p ;
     int  ans ;

     pinMode(SWpin,INPUT_PULLUP) ; // スイッチ接続の端子、プルアップ入力で設定
     pinMode(10, OUTPUT) ;         // この行がないとSDライブラリが動作しないらしい
     
     Serial.begin(9600) ;
//     // ＳＤカードの初期化処理(フォーマットではないよ)
//     if (!SD.begin(SDCSpin)) {
//         // カードの初期化に失敗したか、またはＳＤが入っていない
//         Serial.println("SD card failed, or not present") ;
//         return ;                  // 終了
//     }
//     Serial.println("SD card initialized.") ;   // ＳＤカードの初期化完了
     // カメラの初期化処理(ソフトシリアル通信の初期化)
     VC0706.begin(38400) ;
     delay(3000) ;                 // 3秒後に開始

     // GetVersion処理
     p = VC0706.GetVersion() ;
     if (p == 0) ErrorDSP("GetVersion") ;
     else        Serial.println(p) ;
     // GetImageSize処理
     ans = VC0706.GetImageSize() ;
     if (ans == -1) {
          ErrorDSP("GetImageSize") ;
          return ;                 // 終了
     } else Serial.println(&JPG_Size[ans&0x0F][0]) ;

     FileNo = 0 ;                  // 写真番号、写真を撮る度にプラスされる
     // Motion監視を開始する
     ans = VC0706.MotionCtlEnable() ;
     ans = VC0706.SetMotionCtl(VC070X_MOTION_DETECT_ON) ;
     if (ans == -1) {
          ErrorDSP("SetMotionCtl") ;
          return ;                 // 終了
     }
     Serial.println("<<< Start motion monitoring! >>>") ;     
}
/*******************************************************************************
*  繰り返し実行される処理の関数(メインの処理)                                  *
*******************************************************************************/
void loop() {
     int ans ;

     // カメラからのデータ受信を行う(モーション監視)
     ans = VC0706.MotionDetect() ;
     if (ans != -1) {
          // 何か受信したようだ
          if (ans == VC070X_COMM_MOTION_DETECTED) {
               // 動きを検出したぞぉ
               Serial.println("Detected movement!!") ;

               /*  ここで写真を保存するなり警報を出すなりご自由に  */

          } else {
               // 他のコマンドを受信したようだ
               Serial.println("I received another command") ;
               VC0706.flush() ;
          }
     }
}
